### 
#
# Date:    30.09.2020
# Titel:   Geometric Morphometric Analysis of Ceratitidae - Input of data
# Comment: This files contains only the data input for subsequent geometric morphometric (GMM) analysis. It is used to import the data, 
#          gather all metadata, and perform the general Procrustes analysis. There is the option to choose between weighted and un-weighted 
#          shape coordinates. This files always MUST be run before further analysis (file "developmental morphospaces" and "ontogenetic morphospaces"). 
#          At the end of this file the raw X/Y coordinates are processed to weighted OR un-weighted Procrustes shape coordinates.
#          For further information on weighted/un-weighted coordinates see publication Bischof et al. 2020. Ontogenetic development
#
###

#### 0. Packages ####
#install.packages("geomorph")   #GMM analysis
#install.packages("RRPP")       #GMM analysis
#install.packages("Morpho")     #GMM analysis
#install.packages("ggplot2")    #tidyverse: Plot
#install.packages("ggrepel")    #tidyverse: Text aestethics (geom_text_repel & geom_label_repel)
#install.packages("plotly")     #tidyverse: transparency in plots(alpha)
#install.packages("scales")     #tidyverse: Plot aesthetics
#install.packages("dplyr")      #tidyverse: Data manipulation
#install.packages("ggforce")    #tidyverse: 
# install.packages("patchwork") #tidyverse: Combine ggplpts
#install.packages("devtools")
#devtools::install_github("fruciano/GeometricMorphometricsMix", dependencies = TRUE) # Reverse PCA (Visualisation)
#install.packages("rgl")        #interactive 3D Plot
#install.packages("geometry")   #Add a hull in 3D plots


#### 1. Data  #####
setwd("C:/Eva/Texte_Paper/20.07_GMM/Scripts_&_Data")

## 1.1. Data input ####
xdata<- scan("Data_Ceratitidae.tps", what="character")
  
No.LM_tot<-176 #Set number of landmarks per specimen
No.LM<-No.LM_tot/11  # Set number of landmarks per configuration (whorl)


### 1.2. Extract metadata ######
# Extract data from tps file

### Total length of string per specimen
No.items<-length(xdata) 
  
### Calculate number of specimens
n.ind<-No.items/((No.LM_tot*2)+3) #176*2 (Coordinates per specimen) +3 times meta data (number of LMs, file-string, ID)
n<-n.ind*11
  
### Calculate length of string with (M.ind) and without additional data (M)
# Only needed for data extraction
M.ind <-No.items/n.ind; rm(No.items)
M<-(M.ind-3)/11

### Retrieve ID from  string 
## Create vector with ID's 
xdata.ind<-xdata

## Create empty vector (to be filled subsequently)
ID<-c() 
for (i in 1:n.ind){
  ID2<- c(xdata.ind[i*M.ind]) #every 75th entry -->i*M
  ID <- c(ID,ID2)
  rm("ID2")}; rm("xdata.ind") 

### 1.2.1. SEPARATED: Meta data list ####
## Metadata for developmental morphospaces
## Extract metadata from string and compile list  
list <- list(
  ind.pure=rep(as.character(substring(ID,4,9)),each=11),   #Specimen No. without whorl number
  species=rep(as.character(substring(ID,19,25)),each=11),
  genus= rep(as.character(substring(ID,19,20)),each=11),
  Coord=c("X", "Y"))

list$ind=paste(list$ind.pure, seq(0.5,5.5, by=0.5), sep="-")       #Specimen No. with whorl number
list$whorl=rep.int(seq(0.5,5.5, by=0.5), times=n.ind)              #Whorl number
list$ind.pure.species=paste(list$ind.pure, list$species, sep="-")  #Specimen No. with species
list$No.species = length(unique(list$species))

#Loop for counting the occurrence of each species (metadata for plots)
list$species.fac<-as.numeric(as.factor(list$species))              
No.species.beginning <- as.numeric(length(unique(list$species)))
for (i in 1:n){
  for(j in 1:No.species.beginning){
    if(list$species.fac[i]==j){list$species.occ[i]<-paste("N=",(length(which(list$species.fac ==j))/11),sep = "")}}}
rm(No.species.beginning)

list$species.occ=paste(list$species, list$species.occ, sep="; ")


## Check if no individual is listed twice (TRUE=OK)
length(list$ind.pure)/11==length(unique(list$ind.pure)) 

# Delete unnecessary elements from xdata
for (i in 1:n.ind){
  xdata<-xdata[-(((i-1)*(No.LM_tot*2))+1)] # Delete every first element (LM=)
  xdata<-xdata[-(i*(No.LM_tot*2)+1)]       # Delete every second last (IMAGE=)
  xdata<-xdata[-(i*(No.LM_tot*2)+1)]       # Delete every last element (ID=)
}


### 1.3. Mirror ##### 
# Mirror every second configuration 

#all x values of even whorls   
odd_whorls<-rep(1,No.LM*2)
even_whorls<-rep(c(-1,1),No.LM)
Conf.Mirror<- c(rep(c(odd_whorls,even_whorls), 5), odd_whorls)
Mirror<-rep(Conf.Mirror, n.ind)
  
xdata<-as.numeric(xdata)*Mirror

#Check  
length(xdata)==length(Mirror)
rm(Mirror); rm(even_whorls); rm(odd_whorls); rm(Conf.Mirror)
  

### 1.4. Data array #####

## Open and empty array with correct no. of dimensions: No.LM=16 LMs, 2 dimension (x and y) and n=72 individuals
A<-array(NA, dim=c(No.LM,2,n))  #n: number of configurations) 
for (i in 1:n){
    A[,,i]<-matrix(xdata[(((i-1)*(M))+1):(i*(M))],
                   No.LM,
                   2,
                   byrow=T)}

# Have a look at it; Mirror correct?
A[,,11]
A[,,12] 
A[,,13] # Configuration 13 is the 2nd configuration of the 2nd specimen: X-values should be negative


### 1.5. Input Procrustes analysis ####

## Define sliding of Landmarks
slide_ind <- c(7,8,11:16)
curves_list <- list(UmbR = c(5,15,3),
                    KeelR = c(2,7,9),
                    KeelL = c(2,8,10),
                    VentR = c(3,13,11, 9),
                    VentL = c(4,14,12,10),
                    UmbL = c(6,16,4))

## Define paired LMs
right <- c(5,15,3,13,11, 9,7) 
left  <- c(6,16,4,14,12,10,8)# important: keep same order
pairedLM <- cbind(left,right); rm(left, right)

## Wireframe (for plots)
wireframe=list(c(1,5,15,3,13,11,9,7,2,8,10,12,14,4,16,6,1))
wireframe.ma=matrix(c(1,5,5,15,15,3,3,13,13,11,11,9,9,7,7,2,2,8,8,10,10,12,12,14,14,4,4,16, 16,6,6,1), ncol=2, byrow = TRUE)

### 1.6. Lists for downstream analysis ####
Plots<-list()
df<-list()

### 1.7 Define color palette for plots ####
Palette=c("turquoise1", "sienna4","tomato","hotpink3","mediumvioletred",   # BiCorde,  BrVogde, DiLawso, FrNevad, FrOccid
          "yellowgreen", "forestgreen","darkslategrey","mediumseagreen",   # GyBlake, GyMimet,   GyRotel,GyWeitsch
          "goldenrod","orangered1", "firebrick4")                          # MaSpini, PaDunni,  PaMeeki

### 1.7 Define shape palette for plots ####
Shapes=c( 8, 22, 24, 23, 23,     # BiCorde,  BrVogde, DiLawso, FrNevad, FrOccid
          25, 25, 25, 25,        # GyBlake, GyMimet,   GyRotel,GyWeitsch
          12, 21, 21)            # MaSpini, PaDunni,  PaMeeki)




#### 2. Combine Subsets ####
# The function combine.subsets requires a different data structure
# Before the function in used a separate GPA on every whorl is performed

### 2.1. Rearrange data set ####
## Data needs to be sorted by whorl
## Create empty arrays that are filled subsequently
{whorl=list()
whorl$`0.5`<-array(NA, dim=c(No.LM,2,n/11))  #n: number of configurations) 
whorl$`1.0`<-array(NA, dim=c(No.LM,2,n/11)) 
whorl$`1.5`<-array(NA, dim=c(No.LM,2,n/11)) 
whorl$`2.0`<-array(NA, dim=c(No.LM,2,n/11)) 
whorl$`2.5`<-array(NA, dim=c(No.LM,2,n/11))
whorl$`3.0`<-array(NA, dim=c(No.LM,2,n/11)) 
whorl$`3.5`<-array(NA, dim=c(No.LM,2,n/11))
whorl$`4.0`<-array(NA, dim=c(No.LM,2,n/11)) 
whorl$`4.5`<-array(NA, dim=c(No.LM,2,n/11))
whorl$`5.0`<-array(NA, dim=c(No.LM,2,n/11)) 
whorl$`5.5`<-array(NA, dim=c(No.LM,2,n/11))}

## Fill up the arrays
## j: number of whorl
{j=1
for (i in 1:(n/11)){
  whorl$`0.5`[,,i]<-matrix(A[,,(j-1)+1+(i-1)*11], #fill from i*1.number to i*last number
                           No.LM,
                           2,
                           byrow=F)
  }
  
  j=2
  for (i in 1:(n/11)){
    whorl$`1.0`[,,i]<-matrix(A[,,(j-1)+1+(i-1)*11], #fill from i*1.number to i*last number
                             No.LM,
                             2,
                             byrow=F)
  }
  
  j=3
  for (i in 1:(n/11)){
    whorl$`1.5`[,,i]<-matrix(A[,,(j-1)+1+(i-1)*11], #fill from i*1.number to i*last number
                             No.LM,
                             2,
                             byrow=F)
  }
  
  j=4
  for (i in 1:(n/11)){
    whorl$`2.0`[,,i]<-matrix(A[,,(j-1)+1+(i-1)*11], #fill from i*1.number to i*last number
                             No.LM,
                             2,
                             byrow=F)
  }
  
  j=5
  for (i in 1:(n/11)){
    whorl$`2.5`[,,i]<-matrix(A[,,(j-1)+1+(i-1)*11], #fill from i*1.number to i*last number
                             No.LM,
                             2,
                             byrow=F)
  }
  
  j=6
  for (i in 1:(n/11)){
    whorl$`3.0`[,,i]<-matrix(A[,,(j-1)+1+(i-1)*11], #fill from i*1.number to i*last number
                             No.LM,
                             2,
                             byrow=F)
  }
  
  j=7
  for (i in 1:(n/11)){
    whorl$`3.5`[,,i]<-matrix(A[,,(j-1)+1+(i-1)*11], #fill from i*1.number to i*last number
                             No.LM,
                             2,
                             byrow=F)
  }
  
  j=8
  for (i in 1:(n/11)){
    whorl$`4.0`[,,i]<-matrix(A[,,(j-1)+1+(i-1)*11], #fill from i*1.number to i*last number
                             No.LM,
                             2,
                             byrow=F)
  }
  
  j=9
  for (i in 1:(n/11)){
    whorl$`4.5`[,,i]<-matrix(A[,,(j-1)+1+(i-1)*11], #fill from i*1.number to i*last number
                             No.LM,
                             2,
                             byrow=F)
  }
  
  j=10
  for (i in 1:(n/11)){
    whorl$`5.0`[,,i]<-matrix(A[,,(j-1)+1+(i-1)*11], #fill from i*1.number to i*last number
                             No.LM,
                             2,
                             byrow=F)
  }
  
  j=11
  for (i in 1:(n/11)){
    whorl$`5.5`[,,i]<-matrix(A[,,(j-1)+1+(i-1)*11], #fill from i*1.number to i*last number
                             No.LM,
                             2,
                             byrow=F)
  }
  }

##Check
head(A[,,length(list$ind)]==whorl$`5.5`[,,(length(list$ind)/11)])


## The function procSym does not allow to slide LMs and to use the symmetry function at the same time
## Therefore the function was first use to slide the LMs and then subsequently with the information on what LMs are symmetric

### 2.2. Sliding LM ####
{
  whorl$`0.5slid`=Morpho::procSym(dataarray = whorl$`0.5`,                                   
                                  scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                                  pairedLM = NULL,SMvector = slide_ind,outlines = curves_list,                           
                                  recursive = TRUE, bending = FALSE, pcAlign = FALSE)
  whorl$`1.0slid`=Morpho::procSym(dataarray = whorl$`1.0`,                                   
                                  scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                                  pairedLM = NULL,SMvector = slide_ind,outlines = curves_list,                           
                                  recursive = TRUE, bending = FALSE, pcAlign = FALSE)
  whorl$`1.5slid`=Morpho::procSym(dataarray = whorl$`1.5`,                                   
                                  scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                                  pairedLM = NULL,SMvector = slide_ind,outlines = curves_list,                           
                                  recursive = TRUE, bending = FALSE, pcAlign = FALSE)
  whorl$`2.0slid`=Morpho::procSym(dataarray = whorl$`2.0`,                                   
                                  scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                                  pairedLM = NULL,SMvector = slide_ind,outlines = curves_list,                           
                                  recursive = TRUE, bending = FALSE, pcAlign = FALSE)
  whorl$`2.5slid`=Morpho::procSym(dataarray = whorl$`2.5`,                                   
                                  scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                                  pairedLM = NULL,SMvector = slide_ind,outlines = curves_list,                           
                                  recursive = TRUE, bending = FALSE, pcAlign = FALSE)
  whorl$`3.0slid`=Morpho::procSym(dataarray = whorl$`3.0`,                                   
                                  scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                                  pairedLM = NULL,SMvector = slide_ind,outlines = curves_list,                           
                                  recursive = TRUE, bending = FALSE, pcAlign = FALSE)
  whorl$`3.5slid`=Morpho::procSym(dataarray = whorl$`3.5`,                                   
                                  scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                                  pairedLM = NULL,SMvector = slide_ind,outlines = curves_list,                           
                                  recursive = TRUE, bending = FALSE, pcAlign = FALSE)
  whorl$`4.0slid`=Morpho::procSym(dataarray = whorl$`4.0`,                                   
                                  scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                                  pairedLM = NULL,SMvector = slide_ind,outlines = curves_list,                           
                                  recursive = TRUE, bending = FALSE, pcAlign = FALSE)
  whorl$`4.5slid`=Morpho::procSym(dataarray = whorl$`4.5`,                                   
                                  scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                                  pairedLM = NULL,SMvector = slide_ind,outlines = curves_list,                           
                                  recursive = TRUE, bending = FALSE, pcAlign = FALSE)
  whorl$`5.0slid`=Morpho::procSym(dataarray = whorl$`5.0`,                                   
                                  scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                                  pairedLM = NULL,SMvector = slide_ind,outlines = curves_list,                           
                                  recursive = TRUE, bending = FALSE, pcAlign = FALSE)
  whorl$`5.5slid`=Morpho::procSym(dataarray = whorl$`5.5`,                                   
                                  scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                                  pairedLM = NULL,SMvector = slide_ind,outlines = curves_list,                           
                                  recursive = TRUE, bending = FALSE, pcAlign = FALSE)
}


### 2.3. Symmetry #####
{
Sym=Morpho::procSym(dataarray = whorl$`0.5slid`$orpdata,                                    
                      scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                      pairedLM = pairedLM,                            
                      recursive = TRUE,bending = FALSE, pcAlign = FALSE)
whorl$`0.5Sym`=Sym$Sym
                      
Sym=Morpho::procSym(dataarray = whorl$`1.0slid`$orpdata,                                    
                      scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                      pairedLM = pairedLM,                            
                      recursive = TRUE,bending = FALSE, pcAlign = FALSE)
whorl$`1.0Sym`=Sym$Sym
                      
Sym=Morpho::procSym(dataarray = whorl$`1.5slid`$orpdata,                                    
                      scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                      pairedLM = pairedLM,                            
                      recursive = TRUE,bending = FALSE, pcAlign = FALSE)
whorl$`1.5Sym`=Sym$Sym
                      
Sym=Morpho::procSym(dataarray = whorl$`2.0slid`$orpdata,                                    
                      scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                      pairedLM = pairedLM,                            
                      recursive = TRUE,bending = FALSE, pcAlign = FALSE)
whorl$`2.0Sym`=Sym$Sym
                      
Sym=Morpho::procSym(dataarray = whorl$`2.5slid`$orpdata,                                    
                      scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                      pairedLM = pairedLM,                            
                      recursive = TRUE,bending = FALSE, pcAlign = FALSE)
whorl$`2.5Sym`=Sym$Sym
                      
Sym=Morpho::procSym(dataarray = whorl$`3.0slid`$orpdata,                                    
                      scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                      pairedLM = pairedLM,                            
                      recursive = TRUE,bending = FALSE, pcAlign = FALSE)
whorl$`3.0Sym`=Sym$Sym
                      
Sym=Morpho::procSym(dataarray = whorl$`3.5slid`$orpdata,                                    
                      scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                      pairedLM = pairedLM,                            
                      recursive = TRUE,bending = FALSE, pcAlign = FALSE)
whorl$`3.5Sym`=Sym$Sym
                      
Sym=Morpho::procSym(dataarray = whorl$`4.0slid`$orpdata,                                    
                      scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                      pairedLM = pairedLM,                            
                      recursive = TRUE,bending = FALSE, pcAlign = FALSE)
whorl$`4.0Sym`=Sym$Sym
                      
Sym=Morpho::procSym(dataarray = whorl$`4.5slid`$orpdata,                                    
                      scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                      pairedLM = pairedLM,                            
                      recursive = TRUE,bending = FALSE, pcAlign = FALSE)
whorl$`4.5Sym`=Sym$Sym
                      
Sym=Morpho::procSym(dataarray = whorl$`5.0slid`$orpdata,                                    
                      scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                      pairedLM = pairedLM,                            
                      recursive = TRUE,bending = FALSE, pcAlign = FALSE)
whorl$`5.0Sym`=Sym$Sym
                      
Sym=Morpho::procSym(dataarray = whorl$`5.5slid`$orpdata,                                    
                      scale = TRUE, reflect = TRUE, CSinit = TRUE, orp = TRUE,
                      pairedLM = pairedLM,                            
                      recursive = TRUE,bending = FALSE, pcAlign = FALSE)
whorl$`5.5Sym`=Sym$Sym
}

### 2.4. Centroid Size #####
## the centroid size is need for the weighting of the function combine.subsets
## The centroid size has to be retrieved from the first GPA
c.size<-c()
for (i in 1:(n/11)){
    DEL<- c(whorl$`0.5slid`$size[i],whorl$`1.0slid`$size[i],whorl$`1.5slid`$size[i],whorl$`2.0slid`$size[i],whorl$`2.5slid`$size[i], whorl$`3.0slid`$size[i],
            whorl$`3.5slid`$size[i],whorl$`4.0slid`$size[i],whorl$`4.5slid`$size[i],whorl$`5.0slid`$size[i],whorl$`5.5slid`$size[i])
    c.size <- c(c.size, DEL)}
  
df$c.size$c.size=c.size
df$c.size$logc.size=log(c.size)
df$c.size$species=list$species
  
## check
df$c.size$c.size[length(list$ind)]==whorl$`5.5slid`$size[(length(list$ind)/11)]
df$c.size$c.size[1]==whorl$`0.5slid`$size[1]
df$c.size$c.size[48]==whorl$`2.0slid`$size[5]


### 2.5. logWeights #####

## Calculate proportional sizes (weights)
# Create df with mean logsizes
df$logweights=data.frame(whorl=c(
  log(mean(whorl$`0.5slid`$size)),log(mean(whorl$`1.0slid`$size)),
  log(mean(whorl$`1.5slid`$size)),log(mean(whorl$`2.0slid`$size)),
  log(mean(whorl$`2.5slid`$size)),log(mean(whorl$`3.0slid`$size)),
  log(mean(whorl$`3.5slid`$size)),log(mean(whorl$`4.0slid`$size)),
  log(mean(whorl$`4.5slid`$size)),log(mean(whorl$`5.0slid`$size)),
  log(mean(whorl$`5.5slid`$size)))); rownames(df$logweights)<-paste("w" , seq(0.5,5.5, by=0.5),sep="")

## Calculate proportional sizes (weights)
#library(dplyr)
df$logweights = dplyr::mutate(df$logweights, 
                              weights = whorl / sum(whorl))
sum(df$logweights$weights)=="1"


### 2.6. Function combine.subsets ##### 
# Create common morphospace =>with weighting if desired
#library(geomorph)
#library(RRPP)

###!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!###
###!!! If a weighted PCA is desired, then change it here (norm.CS) !!!###
###!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!###
comb<-geomorph::combine.subsets(
  w0.5=whorl$`0.5Sym`, w1.0=whorl$`1.0Sym`, w1.5=whorl$`1.5Sym`, w2.0=whorl$`2.0Sym`, w2.5=whorl$`2.5Sym`,
  w3.0=whorl$`3.0Sym`, w3.5=whorl$`3.5Sym`, w4.0=whorl$`4.0Sym`, w4.5=whorl$`4.5Sym`, w5.0=whorl$`5.0Sym`,
  w5.5=whorl$`5.5Sym`,
  gpa=TRUE, 
  PrinAxes = FALSE, #gpagen argument for not aligning specimens (otherwise some config. are rotated)
  norm.CS = TRUE #no weighting: centroid size=1
  #norm.CS = FALSE, weights = df$logweights$weights # Weighting: logWeights

)

### 2.7. Rearrange subsets ##### 
## Put in the original order

Shape.df<-array(NA, dim=c(No.LM,2,n)) # create array with dim=176x2x72
for (i in 1:(n/11)){
  Shape.df[,,(i-1)*11+1]<-comb$coords[1:16,,i]
  Shape.df[,,(i-1)*11+1+1]<-comb$coords[17:32,,i]
  Shape.df[,,(i-1)*11+1+2]<-comb$coords[33:48,,i]
  Shape.df[,,(i-1)*11+1+3]<-comb$coords[49:64,,i]
  Shape.df[,,(i-1)*11+1+4]<-comb$coords[65:80,,i]
  Shape.df[,,(i-1)*11+1+5]<-comb$coords[81:96,,i]
  Shape.df[,,(i-1)*11+1+6]<-comb$coords[97:112,,i]
  Shape.df[,,(i-1)*11+1+7]<-comb$coords[113:128,,i]
  Shape.df[,,(i-1)*11+1+8]<-comb$coords[129:144,,i]
  Shape.df[,,(i-1)*11+1+9]<-comb$coords[145:160,,i]
  Shape.df[,,(i-1)*11+1+10]<-comb$coords[161:176,,i]
}

## Check 
str(Shape.df);str(comb$coords)
head(comb$coords[161:176,,n/11]==Shape.df[,,n])

### 2.8. Plot specimens ####
{
windows(width=12, height=6);par(mfrow=c(1,2), mar=c(1,1,1,1))

#Plot individual whorls or specimens
geomorph::plotAllSpecimens(whorl$"5.5Sym", label=TRUE, links = wireframe.ma, plot.param = list(txt.col="blue", txt.cex=2))+mtext(paste("Shape after sliding & Symmetry", sep=""))

#Plot all specimens of all whorls
geomorph::plotAllSpecimens(Shape.df, label=TRUE, links = wireframe.ma, plot.param = list(pt.cex=0.5, pt.bg=1:16, txt.col=1:16, txt.cex=2))+
  mtext(paste("Shape with separate GPAs", sep=""))
}

